package com.zoniot.ccrc.commom.config;



import com.zoniot.ccrc.commom.exception.ApplicationException;
import com.zoniot.ccrc.commom.model.AjaxMessage;
import com.zoniot.ccrc.commom.model.ResultStatus;
import lombok.extern.slf4j.Slf4j;
import org.springframework.security.access.AccessDeniedException;
import org.springframework.security.authentication.BadCredentialsException;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.validation.BindingResult;
import org.springframework.validation.FieldError;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.annotation.ControllerAdvice;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.validation.ConstraintViolationException;
import java.util.List;

/**
 * 统一异常处理
 */
@Slf4j
@ControllerAdvice
public class ExceptionHandle {
    private AjaxMessage ajaxMessage = new AjaxMessage();

    /**
     * 处理未定义的其他异常信息
     *
     * @param e
     * @return
     */
    @ResponseBody
    @ExceptionHandler(value = Exception.class)
    public AjaxMessage handler(Exception e) {
        if (e instanceof ApplicationException) {
            ApplicationException exception = (ApplicationException) e;
            ajaxMessage.setMsg(exception.getStatus(), exception.getMessage());
            return ajaxMessage;
        } else if (e instanceof AccessDeniedException) {
            ajaxMessage.setMsg(ResultStatus.ACCESS_DENIED_ERROR);
            return ajaxMessage;
        } else if (e instanceof UsernameNotFoundException) {
            ajaxMessage.setMsg(ResultStatus.USERNAME_NOT_FOUND_ERROR);
            return ajaxMessage;
        } else if (e instanceof BadCredentialsException) {
            ajaxMessage.setMsg(ResultStatus.USERNAME_NOT_FOUND_ERROR);
            return ajaxMessage;
        }  else {
            ajaxMessage.setMsg(ResultStatus.SYSTEM_ERROR);
            log.error("[系统异常] {}", e);
            return ajaxMessage;
        }
    }

    /**
     * validation
     * 处理请求对象属性不满足校验规则的异常信息
     *
     * @param exception
     * @return
     * @throws Exception
     */
    @ResponseBody
    @ExceptionHandler(value = MethodArgumentNotValidException.class)
    public AjaxMessage exception(MethodArgumentNotValidException exception) {
        BindingResult result = exception.getBindingResult();
        final List<FieldError> fieldErrors = result.getFieldErrors();
        StringBuilder builder = new StringBuilder();

        for (FieldError error : fieldErrors) {
            builder.append(error.getDefaultMessage() + ",");
        }
        String str = builder.toString();
        ajaxMessage.setMsg(-102, str.substring(0, str.length() - 1));
        return ajaxMessage;
    }

    /**
     * validation
     * 处理请求单个参数不满足校验规则的异常信息
     *
     * @param exception
     * @return
     * @throws Exception
     */
    @ResponseBody
    @ExceptionHandler(value = ConstraintViolationException.class)
    public AjaxMessage constraintViolationExceptionHandler(ConstraintViolationException exception) {
        ajaxMessage.setMsg(-102, exception.getMessage());
        return ajaxMessage;
    }

}
