package com.zoniot.ccrc.commom.utils;

import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;

import javax.servlet.http.HttpServletRequest;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.text.SimpleDateFormat;
import java.time.LocalTime;
import java.util.*;

@Slf4j
public class Util {
    /**
     * 获取IP地址
     * @param request
     * @return
     */
    public static String getIpAddr(HttpServletRequest request) {
        String ipAddress = null;
        try {
            ipAddress = request.getHeader("x-forwarded-for");
            //log.info("x-forwarded-for = {}",ipAddress);
            if (ipAddress == null || ipAddress.length() == 0 || "unknown".equalsIgnoreCase(ipAddress)) {
                ipAddress = request.getHeader("Proxy-Client-IP");
                //log.info("Proxy-Client-IP = {}",ipAddress);
            }
            if (ipAddress == null || ipAddress.length() == 0 || "unknown".equalsIgnoreCase(ipAddress)) {
                ipAddress = request.getHeader("WL-Proxy-Client-IP");
                //log.info("WL-Proxy-Client-IP = {}",ipAddress);
            }
            if (ipAddress == null || ipAddress.length() == 0 || "unknown".equalsIgnoreCase(ipAddress)) {
                ipAddress = request.getRemoteAddr();
                //log.info("getRemoteAddr = {}",ipAddress);
                if (ipAddress.equals("127.0.0.1")) {
                    // 根据网卡取本机配置的IP
                    InetAddress inet = null;
                    try {
                        inet = InetAddress.getLocalHost();
                    } catch (UnknownHostException e) {
                        e.printStackTrace();
                    }
                    ipAddress = inet.getHostAddress();
                    //log.info("getHostAddress = {}",ipAddress);
                }
            }
            // 对于通过多个代理的情况，第一个IP为客户端真实IP,多个IP按照','分割
            if (ipAddress != null && ipAddress.length() > 15) { // "***.***.***.***".length()
                // = 15
                //log.info("ipAddress = {}",ipAddress);
                if (ipAddress.indexOf(",") > 0) {
                    ipAddress = ipAddress.substring(0, ipAddress.indexOf(","));
                }
            }
        } catch (Exception e) {
            ipAddress="";
        }
        // ipAddress = this.getRequest().getRemoteAddr();
        return ipAddress;
    }


    public static String createUUIDId() {
        UUID uuid = UUID.randomUUID();
        return uuid.toString().replaceAll("-", "");
    }


    public static String changeCronExpression(String cronExpression , LocalTime localTime) {
        String[] expressionArr = StringUtils.split(cronExpression, " ");
        if (expressionArr != null) {
            expressionArr[0] = String.valueOf(localTime.getSecond());//秒
            expressionArr[1] = String.valueOf(localTime.getMinute());//分
            expressionArr[2] = String.valueOf(localTime.getHour());//时
        }
        return StringUtils.join(expressionArr, " ");
    }


    /**
     * 去重
     * @param list
     * @return
     */
    public static List<Integer> removeDuplicate(List<Integer> list) {
        return new ArrayList<Integer>(new HashSet<>(list));
    }



    /*
     *  获取流水号
     * @param type 微信：W  现金、预存账户：C 银行：B 支付宝：A
     * @param meterCode 水表档案号
     * @return
     */
    public static  String getBatchNo(String type, String meterCode) {
        // 设置批次号 用户号+年月日时分秒+5位随机数
        Date currentTime = new Date();
        SimpleDateFormat formatter = new SimpleDateFormat("yyMMddHHmm");
        String dateString = formatter.format(currentTime);
        Random random = new Random();
        int rannum = (int) (random.nextDouble() * (99999 - 10000 + 1)) + 10000;
        String batchNoString =type + meterCode + dateString + rannum;
        return batchNoString;
    }

    public static List<Integer> difference(List<Integer> firstArrayList, List<Integer> secondArrayList){
        List<Integer> list1 = new ArrayList<>();
        CollectionUtils.addAll(list1, new Object[firstArrayList.size()]);
        Collections.copy(list1, firstArrayList);

        List<Integer> list2 = new ArrayList<>();
        CollectionUtils.addAll(list2, new Object[secondArrayList.size()]);
        Collections.copy(list2, secondArrayList);

        list1.removeAll(list2);
        return list1;
    }
}
