package com.zoniot.ccrc.controller;

import com.zoniot.ccrc.commom.model.AjaxMessage;
import com.zoniot.ccrc.commom.model.ResultStatus;
import com.zoniot.ccrc.commom.utils.RedisUtil;
import com.zoniot.ccrc.commom.utils.TreeUtil;
import com.zoniot.ccrc.commom.utils.UserUtil;
import com.zoniot.ccrc.dao.SiteUserMapper;
import com.zoniot.ccrc.entity.Permission;
import com.zoniot.ccrc.entity.User;
import com.zoniot.ccrc.service.LoginLogsService;
import com.zoniot.ccrc.service.PermissionService;
import com.zoniot.ccrc.service.UserService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

@Slf4j
@Controller
@ResponseBody
@RequestMapping("user")
@Api(description = "用户信息")
public class UserController {


    @Autowired
    private UserService userService;

    @Autowired
    private PermissionService permissionService;
    @Autowired
    private LoginLogsService loginLogsService;
    @Autowired
    RedisUtil redisUtil;

    @Resource
    private SiteUserMapper siteUserMapper;


    @PostMapping("smsCodeLogin")
    @ApiOperation(value = "手机验证码登录")
    public AjaxMessage smsCodeLogin(
            @ApiParam(value = "手机号码", required = true, defaultValue = "13265537610") @RequestParam(required = true) String mobile,
            @ApiParam(value = "手机验证码", required = true, defaultValue = "1234") @RequestParam(required = true) String smsCode,
            HttpServletRequest httpServletRequest
    ) {

        //这里只写控制层为了提供swagger api, 具体实现逻辑在spring security（com.bz.smart_city.commom.security.smsCode）
        return new AjaxMessage<>(ResultStatus.OK);
    }

    @PostMapping("logout")
    @ApiOperation(value = "退出")
    public AjaxMessage logout() {
        userService.logout();
        return new AjaxMessage(ResultStatus.OK);
    }

    @GetMapping("info")
    @ApiOperation(value = "用户信息")
    public AjaxMessage<User> user() {
        User user = UserUtil.getCurrentUser();
        return new AjaxMessage<>(ResultStatus.OK, user);
    }

    @GetMapping("menus")
    @ApiOperation(value = "菜单")
    public AjaxMessage<List<Permission>> menus() {
        List<Permission> list = permissionService.getMenuTreeByUserId(UserUtil.getCurrentUser().getId());
        //JSONArray array = new JSONArray();
        //TreeUtil.setPermissionsTree(1, list, array);
        return new AjaxMessage<>(ResultStatus.OK, TreeUtil.getMenuTree(list, 1, 1));
    }

    @GetMapping("element")
    @ApiOperation(value = "元素")
    public AjaxMessage<Set<String>> element() {
        List<Permission> list = permissionService.getElementByUserId(UserUtil.getCurrentUser().getId());

        Set<String> set = list.parallelStream().filter(p -> !StringUtils.isEmpty(p.getPermission()))
                .map(Permission::getPermission).collect(Collectors.toSet());
        return new AjaxMessage<>(ResultStatus.OK, set);
    }

    @PostMapping("changePassword")
    @ApiOperation(value = "修改密码")
    public AjaxMessage changePassword(
            @ApiParam(value = "旧密码", required = true) @RequestParam(required = true) String oldPassword,
            @ApiParam(value = "新密码", required = true) @RequestParam(required = true) String newPassword
    ) {
        userService.changePassword(oldPassword, newPassword);
        return new AjaxMessage(ResultStatus.OK);
    }




}
