package com.zoniot.ccrc.service.impl;

import com.zoniot.ccrc.commom.utils.FileUtil;
import com.zoniot.ccrc.dao.FileMapper;
import com.zoniot.ccrc.entity.File;
import com.zoniot.ccrc.service.FileService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.IOException;
import java.time.LocalDateTime;
import java.util.List;

@Slf4j
@Service
public class FileServiceImpl implements FileService {

    @Value("${files.path}")
    private String filesPath;
    @Resource
    private FileMapper fileMapper;

    @Override
    public int insert(File file) {
        return fileMapper.insert(file);
    }

    @Override
    public int insertSelective(File file) {
        return fileMapper.insertSelective(file);
    }

    @Override
    public int insertList(List<File> files) {
        return fileMapper.insertList(files);
    }

    @Override
    public int updateByPrimaryKeySelective(File file) {
        return fileMapper.updateByPrimaryKeySelective(file);
    }

    @Override
	public File getFileInfo(String fileId) {
    	File file = fileMapper.getById(fileId);
		return file;
	}
    
    @Override
    public File uploadFile(MultipartFile file) {
        String fileOrigName = file.getOriginalFilename();
        if (!fileOrigName.contains(".")) {
            throw new IllegalArgumentException("缺少后缀名");
        }

        String md5 = null;
        try {
            md5 = FileUtil.fileMd5(file.getInputStream());
        } catch (IOException e) {
            e.printStackTrace();
        }
        File fileInfo = fileMapper.getById(md5);
        if (fileInfo != null) {
            fileInfo.setUpdateBy("system");
            fileInfo.setUpdateDate(LocalDateTime.now());
            fileMapper.updateByPrimaryKeySelective(fileInfo);
            return fileInfo;
        }

        fileOrigName = fileOrigName.substring(fileOrigName.lastIndexOf("."));
        String pathname = FileUtil.getPath() + md5 + fileOrigName;
        String fullPath = filesPath + pathname;
        FileUtil.saveFile(file, fullPath);

        long size = file.getSize();
        String contentType = file.getContentType();

        fileInfo = new File();
        fileInfo.setId(md5);
        fileInfo.setFileType(contentType);
        fileInfo.setName(file.getName());
        fileInfo.setSize(size);
        fileInfo.setPath(fullPath);
        fileInfo.setUrl(pathname);
        fileInfo.setType(contentType.startsWith("image/") ? 1 : 0);
        fileInfo.setStatus(1);
        fileInfo.setCreateBy("system");
        fileInfo.setCreateDate(LocalDateTime.now());
        fileMapper.insert(fileInfo);
        return fileInfo;
    }

}
